/*********************************************************************************************************
*
* @File                : TouchPanel.c
* @Hardware Environment: 
* @Build Environment   : RealView MDK-ARM  Version: 4.54
* @Version             : V1.0
* @date                : 201231221
* @Author              :jason
*
*                                  (c) Copyright 2005-2011, WaveShare
*                                       http://www.waveshare.net
*                                          All Rights Reserved
*
*********************************************************************************************************/

#include "Touchpanel.h"
#include "LCD22.h"


Coordinate  display ;




/* Private define ------------------------------------------------------------*/
#define THRESHOLD 2



/*******************************************************************************
* Function Name  : TP_GetAdXY
* Description    : Read ADS7843
* Input          : None
* Output         : None
* Return         : 
* Attention		 : None
*******************************************************************************/
unsigned long GetTouchADC (unsigned char data)
{	  
	unsigned long NUMH , NUML;
	unsigned long Num;
	
	LCD_CS_H();
	TOUCH_nCS_L();
	SPI1_Send_Data(data); 
	NUMH=SPI1_Send_Data(0x00);
	NUML=SPI1_Send_Data(0x00);
	Num=((NUMH)<<8)+NUML; 	
	Num>>=4;                
	TOUCH_nCS_H();

	return(Num);   
}
/*******************************************************************************
* Function Name  : TP_GetAdXY
* Description    : Read ADS7843
* Input          : None
* Output         : None
* Return         : 
* Attention		 : None
*******************************************************************************/
void TP_GetAdXY(unsigned int *x,unsigned int *y)  
{  
  unsigned long adx,ady; 
  adx=GetTouchADC(CHX);  
  ady=GetTouchADC(CHY); 
  *x=adx; 
  *y=ady; 
} 


/*******************************************************************************
* Function Name  : TP_DrawPoint
* Description    : 
* Input          : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate 
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void TP_DrawPoint(unsigned int Xpos,unsigned int Ypos,unsigned int color)
{
  LCD_SetPoint(Xpos,Ypos,color);     /* Center point */
  LCD_SetPoint(Xpos+1,Ypos,color);
  LCD_SetPoint(Xpos,Ypos+1,color);
  LCD_SetPoint(Xpos+1,Ypos+1,color);	
	
}	

/*******************************************************************************
* Function Name  : DrawCross
* Description    : 
* Input          : - Xpos: Row Coordinate
*                  - Ypos: Line Coordinate 
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/
void DrawCross(unsigned int Xpos,unsigned int Ypos,unsigned int color)
{
  LCD_DrawLine(Xpos-15,Ypos,Xpos-2,Ypos,color);
  LCD_DrawLine(Xpos+2,Ypos,Xpos+15,Ypos,color);
  LCD_DrawLine(Xpos,Ypos-15,Xpos,Ypos-2,color);
  LCD_DrawLine(Xpos,Ypos+2,Xpos,Ypos+15,color);
  
  LCD_DrawLine(Xpos-15,Ypos+15,Xpos-7,Ypos+15,color);
  LCD_DrawLine(Xpos-15,Ypos+7,Xpos-15,Ypos+15,color);

  LCD_DrawLine(Xpos-15,Ypos-15,Xpos-7,Ypos-15,color);
  LCD_DrawLine(Xpos-15,Ypos-7,Xpos-15,Ypos-15,color);

  LCD_DrawLine(Xpos+7,Ypos+15,Xpos+15,Ypos+15,color);
  LCD_DrawLine(Xpos+15,Ypos+7,Xpos+15,Ypos+15,color);

  LCD_DrawLine(Xpos+7,Ypos-15,Xpos+15,Ypos-15,color);
  LCD_DrawLine(Xpos+15,Ypos-15,Xpos+15,Ypos-7,color);	  	
}	
	
/*******************************************************************************
* Function Name  : Read_Ads7846
* Description    : Get TouchPanel X Y
* Input          : None
* Output         : None
* Return         : Coordinate *
* Attention		 : None
*******************************************************************************/
Coordinate *Read_Ads7846(void)
{
  static Coordinate  screen;
  unsigned int m0,m1,m2,TP_X[1],TP_Y[1],temp[3];
  unsigned char count=0;
  int buffer[2][9]={{0},{0}};
  
  do
  {		   
    TP_GetAdXY(TP_X,TP_Y);  
    buffer[0][count]=TP_X[0];  
    buffer[1][count]=TP_Y[0];
    count++;  
  }

  while(!TP_INT_IN&& count<9);  /* TP_INT_IN  */
  if(count==9)   /* Average X Y  */ 
  {
	/* Average X  */
    temp[0]=(buffer[0][0]+buffer[0][1]+buffer[0][2])/3;
	temp[1]=(buffer[0][3]+buffer[0][4]+buffer[0][5])/3;
	temp[2]=(buffer[0][6]+buffer[0][7]+buffer[0][8])/3;

	m0=temp[0]-temp[1];
	m1=temp[1]-temp[2];
	m2=temp[2]-temp[0];

	m0=m0>0?m0:(-m0);
  m1=m1>0?m1:(-m1);
	m2=m2>0?m2:(-m2);

	if( m0>THRESHOLD  &&  m1>THRESHOLD  &&  m2>THRESHOLD ) return 0;

	if(m0<m1)
	{
	  if(m2<m0) 
	    screen.x=(temp[0]+temp[2])/2;
	  else 
	    screen.x=(temp[0]+temp[1])/2;	
	}
	else if(m2<m1) 
	  screen.x=(temp[0]+temp[2])/2;
	else 
	  screen.x=(temp[1]+temp[2])/2;

	/* Average Y  */
  temp[0]=(buffer[1][0]+buffer[1][1]+buffer[1][2])/3;
	temp[1]=(buffer[1][3]+buffer[1][4]+buffer[1][5])/3;
	temp[2]=(buffer[1][6]+buffer[1][7]+buffer[1][8])/3;
	m0=temp[0]-temp[1];
	m1=temp[1]-temp[2];
	m2=temp[2]-temp[0];
	m0=m0>0?m0:(-m0);
	m1=m1>0?m1:(-m1);
	m2=m2>0?m2:(-m2);
	if(m0>THRESHOLD&&m1>THRESHOLD&&m2>THRESHOLD) return 0;

	if(m0<m1)
	{
	  if(m2<m0) 
	    screen.y=(temp[0]+temp[2])/2;
	  else 
	    screen.y=(temp[0]+temp[1])/2;	
    }
	else if(m2<m1) 
	   screen.y=(temp[0]+temp[2])/2;
	else
	   screen.y=(temp[1]+temp[2])/2;

	return &screen;
  }  
  return 0; 
}

	
void Color_Box( unsigned int Xpos,unsigned int Ypos, unsigned int X_Width, unsigned int Y_Width,unsigned int bkColor, unsigned int Stroke_Color )
{
	unsigned long i,j;

	if( Xpos >= MAX_X || Ypos >= MAX_Y )
	{
		return;
	}	
#if  ( DISP_ORIENTATION == 0 ) 
	TOUCH_nCS_H();
	LCD_WriteReg(0x210,Xpos); 	//x start point
	LCD_WriteReg(0x212,Ypos); 	//y start point
	LCD_WriteReg(0x211,Xpos+X_Width);	//x end point
	LCD_WriteReg(0x213,Ypos+Y_Width);	//y end point

	LCD_WriteReg(0x200,Xpos);	//RAM X address(0 ~ FF, actually 0 ~ EF)	
	LCD_WriteReg(0x201,Ypos);	//RAM Y address(0 ~ 1FF, actually 0 ~ 13F)
#elif  ( DISP_ORIENTATION == 90 )
	TOUCH_nCS_H();
	LCD_WriteReg(0x212,Xpos); 	//x start point
	LCD_WriteReg(0x210,Ypos); 	//y start point
	LCD_WriteReg(0x213,Xpos+X_Width);	//x end point
	LCD_WriteReg(0x211,Ypos+Y_Width);	//y end point
	LCD_WriteReg(0x201,Xpos);	//RAM X address(0 ~ FF, actually 0 ~ EF)	
	LCD_WriteReg(0x200,Ypos);	//RAM Y address(0 ~ 1FF, actually 0 ~ 13F)
	
#elif  ( DISP_ORIENTATION == 180 )	
	TOUCH_nCS_H();
	LCD_WriteReg(0x211,239-Xpos); 	//x start point
	LCD_WriteReg(0x213,319-Ypos); 	//y start point
	LCD_WriteReg(0x210,239-(Xpos+X_Width));	//x end point
	LCD_WriteReg(0x212,319-(Ypos+Y_Width));	//y end point

	LCD_WriteReg(0x200,239-Xpos);	//RAM X address(0 ~ FF, actually 0 ~ EF)	
	LCD_WriteReg(0x201,319-Ypos);	//RAM Y address(0 ~ 1FF, actually 0 ~ 13F)
#elif  ( DISP_ORIENTATION == 270 )	
	TOUCH_nCS_H();
	LCD_WriteReg(0x213,319-Xpos); 	//x start point
	LCD_WriteReg(0x210,Ypos); 	//y start point
	LCD_WriteReg(0x212,319-(Xpos+X_Width));	//x end point
	LCD_WriteReg(0x211,Ypos+Y_Width);	//y end point

	LCD_WriteReg(0x201,319-Xpos);	//RAM X address(0 ~ FF, actually 0 ~ EF)	
	LCD_WriteReg(0x200,Ypos);	//RAM Y address(0 ~ 1FF, actually 0 ~ 13F) 	
	
#endif	

	LCD_WriteIndex(0x202);	//RAM Write index
	LCD_CS_L();
	LCD_RS_H();
	
	for( i = 0; i < X_Width * Y_Width; i++ )
	{
		LCD_WriteData(bkColor);
	}

	
// 	for(i=Xpos;i<X_Width+Xpos;i++)
// 		for(j=Ypos;j<Y_Width+Ypos;j++)
// 	LCD_SetPoint(i,j,bkColor);
// 	
	LCD_DrawLine(Xpos,Ypos,Xpos+X_Width,Ypos,Stroke_Color);
	LCD_DrawLine(Xpos,Ypos+1,Xpos+X_Width,Ypos+1,Stroke_Color);
	
	LCD_DrawLine(Xpos,Ypos,Xpos,Ypos+Y_Width,Stroke_Color);
	LCD_DrawLine(Xpos+1,Ypos,Xpos+1,Ypos+Y_Width,Stroke_Color);
	
	LCD_DrawLine(Xpos,Ypos+Y_Width,Xpos+X_Width,Ypos+Y_Width,Stroke_Color);
	LCD_DrawLine(Xpos,Ypos+Y_Width-1,Xpos+X_Width,Ypos+Y_Width-1,Stroke_Color);
	
	LCD_DrawLine(Xpos+X_Width,Ypos+Y_Width,Xpos+X_Width,Ypos,Stroke_Color);
	LCD_DrawLine(Xpos-1+X_Width,Ypos+Y_Width,Xpos-1+X_Width,Ypos,Stroke_Color);
} 
unsigned int Table_Color[]={White,Black,Grey,Blue,Blue2,Red,Magenta,Green,Cyan,Yellow,Blue2};

void Disp_Touchpanel(void)
{
  unsigned int i;
  GUI_Text(0,0,"Touchpad:",White,Blue);
  
  for(i=0;i<11;i++)
  Color_Box(21*i,298,21,21,Table_Color[i],Blue2);
  GUI_Text(215,300,"C",Red,Blue2);
  
  LCD_DrawLine(0,16,0+239,16,Blue2);
  LCD_DrawLine(0,16+1,0+239,16+1,Blue2);
  
  LCD_DrawLine(0,16,0,16+274,Blue2);
  LCD_DrawLine(0+1,16,0+1,16+274,Blue2);
  
  LCD_DrawLine(0,16+274,0+239,16+274,Blue2);
  LCD_DrawLine(0,16+274-1,0+239,16+274-1,Blue2);
  
  LCD_DrawLine(0+239,16+274,0+239,16,Blue2);
  LCD_DrawLine(0-1+239,16+274,0-1+239,16,Blue2);
}
/*******************************************************************************
* Function Name  : Touch_Debug
* Description    : 
* Input          : None
* Output         : None
* Return         : None
* Attention		 : None
*******************************************************************************/

void Touch_Debug(void)
{
	unsigned long XAD; 
	XAD=32;
	GUI_Text(0,0,"Coordinate:",Red,Grey);
	GUI_Text(0,16,"XAD=",Red,Grey);
	GUI_Text(0,32,"yAD=",Red,Grey); 
	/*x*/
	PutChar(XAD,16,48+display.x/1000,Black,Grey); 			XAD=XAD+8;
	PutChar(XAD,16,48+display.x%1000/100,Black,Grey);  XAD=XAD+8; 
	PutChar(XAD,16,48+display.x%100/10,Black,Grey); 	  XAD=XAD+8;
	PutChar(XAD,16,48+display.x%10,Black,Grey);     		XAD=XAD+8;
	/*y*/
	XAD=32;
	PutChar(XAD,32,48+display.y/1000,Black,Grey);   		 XAD=XAD+8;
	PutChar(XAD,32,48+display.y%1000/100,Black,Grey);   XAD=XAD+8;
	PutChar(XAD,32,48+display.y%100/10,Black,Grey);     XAD=XAD+8;
	PutChar(XAD,32,48+display.y%10,Black,Grey);         XAD=XAD+8;

}

